﻿Omniscience = LibStub("AceAddon-3.0"):NewAddon("Omniscience", "AceEvent-3.0", "LibBars-1.0")
local bars = LibStub("LibBars-1.0")
local media = LibStub("LibSharedMedia-3.0")
local bitband = bit.band

local defaults = {
	profile = {
	texture = "Ruben",
	font = "ABF",
	fontsize = 9,
	playeronly = true,
	groups = { 
		cooldown_friendly = {name = "Group Cooldowns",   srcflags = 7,   dstflags = nil, enabled = true, position = {p="CENTER", rp="CENTER", px=300, py=100}},
		cooldown_hostile =  {name = "Hostile Cooldowns", srcflags = 8,   dstflags = nil, enabled = true, position = {p="CENTER", rp="CENTER", px=300, py=150}},
		cast_hostile =      {name = "Hostile Casts",     srcflags = 8,   dstflags = nil, enabled = true, position = {p="CENTER", rp="CENTER", px=300, py=200}},
		buff_hostile =      {name = "Hostile Buffs",     srcflags = 8,   dstflags = nil, enabled = true, position = {p="CENTER", rp="CENTER", px=300, py=250}},
		crowd_control =     {name = "Crowd Control",     srcflags = nil, dstflags = 8,   enabled = true, position = {p="CENTER", rp="CENTER", px=300, py=300}},
	},
    },
}

function Omniscience:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("OmniscienceDB", defaults, "Default")
	for k in pairs(self.db.profile.groups) do
		local group = k
		self[group] = self:NewBarGroup(self.db.profile.groups[group].name, nil, 200, 12)
		self[group]:SetTexture(media:Fetch("statusbar", self.db.profile.texture))
		self[group]:SetFont(media:Fetch("font", self.db.profile.font, self.db.profile.fontsize))
		self[group]:SetFlashPeriod(0)
		self[group].RegisterCallback(self, "AnchorClicked")
		self[group]:SetColorAt(0.00, 1, 0, 0, 1)
		self[group]:SetColorAt(0.33, 1, 1, 0, 1)
		self[group]:SetColorAt(0.66, 0, 1, 1, 1)
		self[group]:SetColorAt(1.00, 0, 0, 1, 1)
		self[group]:SetPoint(self.db.profile.groups[group].position.p, UIParent, self.db.profile.groups[group].position.rp, self.db.profile.groups[group].position.px, self.db.profile.groups[group].position.py)
		self[group]:SetScale(0.75)
		self[group].id = group
		self[group]:HideAnchor()
	end
end

function Omniscience:UpdateMedia()
	for k in pairs(self.db.profile.groups) do
		self[k]:SetTexture(media:Fetch("statusbar", self.db.profile.texture))
		self[k]:SetFont(media:Fetch("font", self.db.profile.font, self.db.profile.fontsize))

	end
end

function Omniscience:OnEnable()
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	media.RegisterCallback(self, "LibSharedMedia_Registered", "UpdateMedia")  
	media.RegisterCallback(self, "LibSharedMedia_SetGlobal", "UpdateMedia")  
	self:UpdateMedia()
end

function Omniscience:OnDisable()
	self:UnRegisterAllEvents()
	media.UnregisterCallback(self)
	for k in pairs(self.db.profile.groups) do
		local group = k
		self[group].UnregisterCallback(self)
	end
end

function Omniscience:AnchorClicked(callback, group, button)
	if button == "RightButton" then
		local p,_,rp,px,py  = group:GetPoint()
		local id = group.id
		self.db.profile.groups[id].position.p = p
		self.db.profile.groups[id].position.rp = rp
		self.db.profile.groups[id].position.px = px
		self.db.profile.groups[id].position.py = py
		group:HideAnchor()
	end
end

function Omniscience:ToggleAnchors()
	for k in pairs(self.db.profile.groups) do
		local group = k
		self[group]:ToggleAnchor()
	end
end

function Omniscience:RunTest()
	for k in pairs(self.db.profile.groups) do
		local group = k
		local icon = select(3, GetSpellInfo(43308))
		self[group]:NewTimerBar("test-"..group, "Test", 30, nil, icon)
	end
end

function Omniscience:CheckAffiliation(flags, match)
	local result = (bitband(flags, match) ~= 0)
	return result or nil
end

function Omniscience:COMBAT_LOG_EVENT_UNFILTERED(event, timestamp, eventType, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, spellID, spellName, spellFlags, ...)
	if eventType == "SPELL_INTERRUPT" then
		if not self.spells["SPELL_CAST_START"][spellID] then return end
		local group = self.spells["SPELL_CAST_START"][spellID].group
		local interrupted = select(1, ...)
		local name = "Omniscience-"..dstGUID.."-"..interrupted
		local bar = self[group]:GetBar(name)
		if bar then self[group]:ReleaseBar(name) end
	return end

	if eventType == "SPELL_AURA_REMOVED" then
		if not self.spells["SPELL_AURA_APPLIED"][spellID] then return end
		local group = self.spells["SPELL_AURA_APPLIED"][spellID].group
		local name = "Omniscience-"..dstGUID.."-"..spellID
		local bar = self[group]:GetBar(name)
		if bar then self[group]:ReleaseBar(name) end
	return end

	if not self.spells[eventType] then return end
	if not self.spells[eventType][spellID] then return end
	local flags, match, actor, actorGUID
	if self.spells[eventType][spellID].srcflags then
		flags, match = srcFlags, self.spells[eventType][spellID].srcflags
		actor, actorGUID = srcName, srcGUID
	elseif self.spells[eventType][spellID].dstflags then
		flags, match = dstFlags, self.spells[eventType][spellID].dstflags
		actor, actorGUID = dstName, dstGUID
	end
	if not actor or not actorGUID or not flags or not match then return end
	if not self:CheckAffiliation(flags, match) then return end

	local name = "Omniscience-"..actorGUID.."-"..spellID
	local value = self.spells[eventType][spellID].time
	local text = actor..": "..spellName
	local icon = select(3, GetSpellInfo(spellID)) or nil
	local group = self.spells[eventType][spellID].group

	self[group]:NewTimerBar(name, text, value, nil, icon)
end
